# -*- coding: utf-8 -*-

#2a) Un elemento di una list Python può essere a sua volta una lista, dando 
# luogo così a "liste annidate". L'operazione di flattening di una lista 
# potenzialmente annidata corrisponde a generare una lista che abbia gli stessi 
# elementi nel medesimo ordine, ma senza nessun annidamento.
# Es. l = [1,2,[3,[4,5],[6],7]] --flattening--> [1, 2, 3, 4, 5, 6, 7]
# Si scriva una funzione di nome "flatten" che restituisca il flattening della lista
# ricevuta in input.
# Es. 
# l = [1,2,[3,[4,5],[6],7]]
# flatten(l) # -> [1, 2, 3, 4, 5, 6, 7]
#
# Nota: sebbene non indispensabile, potrebbe essere utile far riferimento alle 
# "Abstract Base Classes" del modulo "collections" 


def flatten(l):
    """
    To return a flattened list, given the target one.
    This is a plain recursive solution that deals with lists only.
    """ 
    res = []
    for elem in l:
        if type(elem) == list:
            res.extend(flatten(elem))
        else:
            res.append(elem)
    return res



## "inline" version
#def flatteninline(l):
#    """
#    To modify inline a list, flattening it (passed as argument).
#    This is a plain recursive solution that deals with lists only.
#    """ 
#    
#    i=0
#    try:
#        while True:
#            if type(l[i]) == list:
#                flatteninline(l[i])
#                ins = l.pop(i)
#                for e in ins:
#                    l.insert(i,e)
#                    i+=1
#                continue
#            i+=1
#    except IndexError:
#        pass

## alternative solutions, more general, dealing also with strings as seqs
#
#import collections
#
#def flatgenerator(l):
#    """
#    A generator to iterate on a nested sequence.
#    """ 
#    for elem in l:
#        if isinstance(elem, collections.Iterable):
#            if isinstance(elem, basestring):
#                for c in elem:
#                    yield c
#            else:
#                for e in flatgenerator(elem):
#                    yield e
#        else:
#            yield elem
#
#
#def flatten2(target):
#    """
#    To return a flattened sequence, given the target one.
#    This solution makes use of the "flat" iterator from "flatgenerator".
#    """                  
#    return type(target)(flatgenerator(target))
#
#    
#
#def flatten3(l):
#    """
#    To return a flattened sequence, given the target one.
#    This is a plain recursive solution.
#    """ 
#
#    res=[]
#    for elem in l:
#        if isinstance(elem, collections.Iterable):
#            if isinstance(elem, basestring):
#                res.extend(list(elem))
#            else:
#                res.extend(flatten3(elem))
#        else:
#            res.append(elem)
#    return res
        